# -*- coding: utf-8 -*-

# PreviewArea.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk, tempfile, os
import const

class PreviewArea(gtk.Image):
	"""
	An area using latex, dvips and Johan Vromans's eps2png to provide a preview for LaTeX source
	"""
	_source = None
	_packages = []
	_changed = False
	_escapeRefs = False
	
	def __init__(self):
		gtk.Image.__init__(self)
		
	def setSource(self, source):
		self._source = source
		self._changed = True
	
	def setPackages(self, packages):
		self._packages = packages
		self._changed = True
	
	def setEscapeRefs(self, settings):
		self._escapeRefs = settings
	
	def render(self):
		if not self._source or not self._changed:
			return
		self._changed = False
		
		source = self._source
		
		if self._escapeRefs:
			# TODO: besser machen (re replace)
			for n in range(9):
				source = source.replace("#%s" % n, "\\#%s" % n)
		
		source = r'\documentclass{article}%s\pagestyle{empty}\begin{document}%s\end{document}' \
				% ("".join(["\usepackage{%s}" % p for p in self._packages]), source)
		# write to temporary file
		temp_file = tempfile.NamedTemporaryFile(mode="w", suffix=".tex")
		temp_file.write(source)
		temp_file.flush()
		# truncate suffix
		filename = os.path.splitext(temp_file.name)[0]
		# render source
		os.system('latex -interaction=batchmode -halt-on-error -output-directory "%s" "%s.tex" 1>/dev/null && dvips -D 120 -q -E -o "%s.eps" "%s.dvi" && %s/eps2png -pngalpha -resolution=120 "%s.eps"' % (os.path.dirname(filename), filename, filename, filename, const.PATH, filename))
		# load image
		self.set_from_file(filename + ".png")
		# cleanup
		temp_file.close()
		for suffix in [".dvi", ".eps", ".png"]:
			try:
				os.unlink(filename + suffix)
			except OSError:
				pass
