# -*- coding: utf-8 -*-

# QuickstartDialog.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk, os
import const

_DOCUMENTCLASSES = {
		"article" : _("Article"),
		"report" : _("Report"),
		"book" : _("Book"),
		"beamer" : _("Beamer slides"),
		"letter" : _("Letter"),
		"scrartcl" : _("Article (KOMA scipt)"),
		"scrreport" : _("Report (KOMA scipt)"),
		"scrbook" : _("Book (KOMA scipt)"),
		"scrlettr" : _("Letter (KOMA scipt)"),
		"scrlttr2" : _("Letter 2 (KOMA script)")
}

_PAPER_SIZES = {
		"a4paper" : "A4",
		"a5paper" : "A5",
		"b5paper" : "B5",
		"executivepaper" : "US-Executive",
		"legalblPaper" : "US-Legal",
		"letterpaper" : "US-Letter"
}

_INPUTENCS = {
		"utf8" : "UTF-8 (Unicode)",
		"ascii" : "US-ASCII",
		"next" : "ASCII (NeXT)",
		"ansinew" : "ASCII (Windows)",
		"applemac" : "ASCII (Apple)",
		"macce" : "MacCE (Apple Central European)",
		"latin1" : "Latin-1",
		"latin2" : "Latin-2",
		"latin3" : "Latin-3 (South European)",
		"latin4" : "Latin-4 (North European)",
		"latin5" : "Latin-5 (Turkish)",
		"latin6" : "Latin-6 (Nordic)",
		"latin7" : "Latin-7 (Baltic)",
		"latin8" : "Latin-8 (Celtic)",
		"latin9" : "Latin-9 (extended Latin-1)",
		"latin10" : "Latin-10 (South-Eastern European)",
		"cp1250" : "CP1250 (Windows Central European)",
		"cp1252" : "CP1252 (Windows Western European)",
		"cp1257" : "CP1257 (Windows Baltic)",
		"cp437" : "CP437 (DOS US)",
		"cp850" : "CP850 (DOS Latin-1)",
		"cp852" : "CP852 (DOS Central European)",
		"cp858" : "CP858 (DOS Western European)",
		"cp865" : "CP865 (DOS Nordic)"
}

_BABELS = {
		"afrikaans" : "Afrikaans",
		"american" : "American",
		"athnum" : "Athnum",
		"austrian" : "Austrian",
		"naustrian" : "Austrian (new spelling)",
		"bahasa" : "Bahasa",
		"basque" : "Basque",
		"breton" : "Breton",
		"british" : "British",
		"bulgarian" : "Bulgarian",
		"catalan" : "Catalan",
		"croatian" : "Croatian",
		"czech" : "Czech",
		"danish" : "Danish",
		"dutch" : "Dutch",
		"english" : "English",
		"UKenglish" : "English (UK)",
		"USenglish" : "English (US)",
		"esperanto" : "Esperanto",
		"estonian" : "Estonian",
		"finnish" : "Finnish",
		"francais" : "Francais",
		"galician" : "Galician",
		"german" : "German",
		"ngerman" : "German (new spelling)",
		"greek" : "Greek",
		"hebrew" : "Hebrew",
		"icelandic" : "Icelandic",
		"interlingua" : "Interlingua",
		"irish" : "Irish",
		"italian" : "Italian",
		"latin" : "Latin",
		"lsorbian" : "Lsorbian",
		"magyar" : "Magyar",
		"norsk" : "Norsk",
		"polish" : "Polish",
		"portuges" : "Portuges",
		"romanian" : "Romanian",
		"russianb" : "Russian",
		"samin" : "Samin",
		"scottish" : "Scottish",
		"serbian" : "Serbian",
		"slovak" : "Slovak",
		"slovene" : "Slovene",
		"spanish" : "Spanish",
		"swedish" : "Swedish",
		"turkish" : "Turkish",
		"ukraineb" : "Ukraine",
		"usorbian" : "Usorbian",
		"welsh" : "Welsh"
}

_LOCALE_TO_BABEL = {
		"en_US" : "american",
		"en_AU" : "english",
		"fr" : "french",
		"it" : "italian",
		"ru" : "russian",
		"de_DE" : "ngerman",
		"de_AU" : "naustrian"
}

_BEAMER_THEMES = ['default', 'Rochester', 'Szeged', 'Darmstadt', 'compatibility', 
		'shadow', 'Hannover', 'Luebeck', 'Madrid', 'sidebar', 'Montpellier', 'Warsaw', 
		'Ilmenau', 'Malmoe', 'Berlin', 'Berkeley', 'lined', 'bars', 'split', 'tree', 
		'Bergen', 'classic', 'PaloAlto', 'Pittsburgh', 'Antibes', 'Dresden', 'plain', 
		'Copenhagen', 'boxes', 'Frankfurt', 'Singapore', 'Goettingen', 'Marburg', 
		'Boadilla', 'JuanLesPins']

class QuickstartDialog(gtk.Dialog):
	def __init__(self, instance):
		self._instance = instance
		
		import locale, getpass, pwd
		
		# store the user's locale code (ie. en_US)
		self.langCode = locale.getdefaultlocale()[0]
		# store a localized date format string for use in time.strftime()
		self.dateFormat = locale.nl_langinfo(locale.D_FMT)
		# get the user's pwd entry
		record = pwd.getpwnam(getpass.getuser())
		if len(record[4]):
			self.username = record[4].split(",")[0]
		else:
			self.username = record[0].title()
		
		# use kpsewhich to search for available document classes, input encodings,
		# babel packages and beamer themes
		if os.system("kpsewhich --version $2>/dev/null") == 0:
			self._availClasses = self._findAvailable(_DOCUMENTCLASSES.keys(), "", ".cls")
			self._availEncodings = self._findAvailable(_INPUTENCS.keys(), "", ".def")
			self._availBabels = self._findAvailable(_BABELS.keys(), "", ".sty")
			if "beamer" in self._availClasses:
				self._availThemes = self._findAvailable(_BEAMER_THEMES, "beamertheme", ".sty")
			else:
				self._availThemes = []
		else:
			self._availClasses = _DOCUMENTCLASSES.keys()
			self._availEncodings = _INPUTENCS.keys()
			self._availBabels = _BABELS.keys()
			self._availThemes = _BEAMER_THEMES
		self._availClasses.sort()
		self._availEncodings.sort()
		self._availBabels.sort()
		
		## build ui
		gtk.Dialog.__init__(self, _("New LaTeX Document"), instance._window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
				(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(10)
		
		## General
		lblGeneral = gtk.Label()
		lblGeneral.set_markup("<b>General</b>")
		lblGeneral.set_alignment(0, .5)
		
		self.entryTitle = gtk.Entry()
		self.entryTitle.set_activates_default(True)
		self.entryAuthor = gtk.Entry()
		self.entryAuthor.set_activates_default(True)
		self.entryDate = gtk.Entry()
		self.entryDate.set_activates_default(True)
		
		lblTitle = gtk.Label(_("_Title:"))
		lblTitle.set_mnemonic_widget(self.entryTitle)
		lblAuthor = gtk.Label(_("_Author:"))
		lblAuthor.set_mnemonic_widget(self.entryAuthor)
		lblDate = gtk.Label(_("_Date:"))
		lblDate.set_mnemonic_widget(self.entryDate)
		
		for label in [lblTitle, lblAuthor, lblDate]:
			label.set_use_underline(True)
			label.set_alignment(0, .5)
		
		## Style and Layout
		lblStyle = gtk.Label()
		lblStyle.set_markup("<b>Style and Layout</b>")
		lblStyle.set_alignment(0, .5)
		
		# document class
		self.cmbClass = gtk.combo_box_new_text()
		for name in self._availClasses:
			self.cmbClass.append_text(_DOCUMENTCLASSES[name])
		
		# beamer theme
		self.cmbTheme = gtk.combo_box_new_text()
		for name in self._availThemes:
			self.cmbTheme.append_text(name)
		
		# font size
		self._adjDefault = gtk.Adjustment(10, 10, 12, 1)
		self._adjBeamer = gtk.Adjustment(11, 8, 20, 1)
		self.spinFontSize = gtk.SpinButton()
		boxFontSize = gtk.HBox()
		boxFontSize.set_spacing(5)
		boxFontSize.pack_start(self.spinFontSize, False)
		lblblFontSizeU = gtk.Label("pt")
		boxFontSize.pack_start(lblblFontSizeU, False)
		
		# paper size
		self.cmbPaperSize = gtk.combo_box_new_text()
		self._availPaperSizes = _PAPER_SIZES.keys()
		for name in self._availPaperSizes:
			self.cmbPaperSize.append_text(_PAPER_SIZES[name])
			
		# paper alignment
		self.cmbPaperAlign = gtk.combo_box_new_text()
		for item in [_("Portrait"), _("Landscape")]:
			self.cmbPaperAlign.append_text(item)
		
		lblClass = gtk.Label(_("Document _Class:"))
		lblClass.set_mnemonic_widget(self.cmbClass)
		self.lblTheme = gtk.Label(_("Th_eme:"))
		self.lblTheme.set_mnemonic_widget(self.cmbTheme)
		lblFontSize = gtk.Label(_("Font _Size:"))
		lblFontSize.set_mnemonic_widget(self.spinFontSize)
		lblPaper = gtk.Label(_("_Paper:"))
		lblPaper.set_mnemonic_widget(self.cmbPaperSize)
		
		for label in [lblClass, self.lblTheme, lblFontSize, lblblFontSizeU, lblPaper]:
			label.set_use_underline(True)
			label.set_alignment(0, .5)
		
		## Language Customization
		lblLanguage = gtk.Label()
		lblLanguage.set_markup("<b>Language Customization</b>")
		lblLanguage.set_alignment(0, .5)
		
		# input encoding
		self.cmbEncoding = gtk.combo_box_new_text()
		for name in self._availEncodings:
			self.cmbEncoding.append_text(_INPUTENCS[name])
		
		lblEncoding = gtk.Label(_("_Input encoding:"))
		lblEncoding.set_use_underline(True)
		lblEncoding.set_mnemonic_widget(self.cmbEncoding)
		lblEncoding.set_alignment(0, .5)
		
		# babel package
		self.cmbBabel = gtk.combo_box_new_text()
		for name in self._availBabels:
			self.cmbBabel.append_text(_BABELS[name])
		
		lblBabel = gtk.Label(_("_Babel package:"))
		lblBabel.set_use_underline(True)
		lblBabel.set_mnemonic_widget(self.cmbBabel)
		lblBabel.set_alignment(0, .5)
		
		## pack
		table = gtk.Table(12, 4)
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		table.set_col_spacing(0, 15)
		table.set_row_spacing(3, 15)
		table.set_row_spacing(8, 15)
		
		table.attach(lblGeneral, 0, 4, 0, 1)
		table.attach(lblTitle, 1, 2, 1, 2)
		table.attach(self.entryTitle, 2, 4, 1, 2)
		table.attach(lblAuthor, 1, 2, 2, 3)
		table.attach(self.entryAuthor, 2, 4, 2, 3)
		table.attach(lblDate, 1, 2, 3, 4)
		table.attach(self.entryDate, 2, 4, 3, 4)
		
		table.attach(lblStyle, 0, 4, 4, 5)
		table.attach(lblClass, 1, 2, 5, 6)
		table.attach(self.cmbClass, 2, 4, 5, 6)
		table.attach(self.lblTheme, 1, 2, 6, 7)
		table.attach(self.cmbTheme, 2, 4, 6, 7)
		table.attach(lblFontSize, 1, 2, 7, 8)
		table.attach(boxFontSize, 2, 4, 7, 8)
		table.attach(lblPaper, 1, 2, 8, 9)
		table.attach(self.cmbPaperSize, 2, 3, 8, 9)
		table.attach(self.cmbPaperAlign, 3, 4, 8, 9)
		
		table.attach(lblLanguage, 0, 4, 9, 10)
		table.attach(lblEncoding, 1, 2, 10, 11)
		table.attach(self.cmbEncoding, 2, 4, 10, 11)
		table.attach(lblBabel, 1, 2, 11, 12)
		table.attach(self.cmbBabel, 2, 4, 11, 12)
		
		self.vbox.add(table)
		
		self.connect("response", self._onResponse)
		self.cmbClass.connect("changed", self._onClassChanged)
		
		self.set_default_response(gtk.RESPONSE_OK)
		
	def run(self):
		self._restoreSettings()
		self.show_all()
		self.set_default_response(gtk.RESPONSE_OK)
		return gtk.Dialog.run(self)
		
	def _restoreSettings(self):
		""" Restore settings from configuration """
		import time
		
		# author
		self.entryAuthor.set_text(self.username)
		# date
		self.entryDate.set_text(time.strftime(self.dateFormat))
		
		# document class
		lastClass = self._instance._prefs.setdefault("/last-document-class", self._availClasses[0])
		try:
			index = self._availClasses.index(lastClass)
		except ValueError:
			index = 0
		self.cmbClass.set_active(index)
		
		# beamer theme
		try:
			lastTheme = self._instance._prefs.setdefault("/last-beamer-theme", self._availThemes[0])
		except IndexError:
			# no beamer themes
			lastTheme = ""
		try:
			index = self._availThemes.index(lastTheme)
		except ValueError:
			index = 0
		self.cmbTheme.set_active(index)
		
		# paper size
		last_size = self._instance._prefs.setdefault("/last-paper-size", self._availPaperSizes[0])
		try:
			index = self._availPaperSizes.index(last_size)
		except ValueError:
			index = 0
		self.cmbPaperSize.set_active(index)
		
		# paper alignment
		self.cmbPaperAlign.set_active(self._instance._prefs.setdefault("/last-paper-alignment", 0))
		
		# input encoding
		last_encoding = self._instance._prefs["/last-input-encoding"]
		if last_encoding:
			# restore the last selected input encoding
			try:
				index = self._availEncodings.index(last_encoding)
			except ValueError:
				index = 0
		else:
			# use Unicode
			try:
				index = self._availEncodings.index("utf8")
			except (IndexError, ValueError):
				index = 0
		self.cmbEncoding.set_active(index)
		
		# babel package
		last_babel = self._instance._prefs["/last-babel-package"]
		if last_babel:
			# restore the last selected babel package
			try:
				index = self._availBabels.index(last_babel)
			except ValueError:
				index = 0
		else:
			# guess a babel package from the user's locale settings
			try:
				babel = _LOCALE_TO_BABEL[self.langCode]
				index = self._availBabels.index(babel)
			except (KeyError, IndexError, ValueError):
				print "Warning: could not map your locale '%s' to a babel package." % self.langCode
				index = 0
		self.cmbBabel.set_active(index)
	
	def _onResponse(self, dialog, response):
		if response == gtk.RESPONSE_OK:
			# store settings
			self._instance._prefs["/last-document-class"] = self._availClasses[self.cmbClass.get_active()]
			if len(self._availThemes):
				self._instance._prefs["/last-beamer-theme"] = self._availThemes[self.cmbTheme.get_active()]
			self._instance._prefs["/last-paper-size"] = self._availPaperSizes[self.cmbPaperSize.get_active()]
			self._instance._prefs["/last-paper-alignment"] = int(self.cmbPaperAlign.get_active())
			self._instance._prefs["/last-input-encoding"] = self._availEncodings[self.cmbEncoding.get_active()]
			self._instance._prefs["/last-babel-package"] = self._availBabels[self.cmbBabel.get_active()]
	
	def _onClassChanged(self, combo):
		documentClass = self._availClasses[self.cmbClass.get_active()]
		setting = (documentClass == "beamer")
		# theme
		self.lblTheme.set_sensitive(setting)
		self.cmbTheme.set_sensitive(setting)
		# font size
		if documentClass == "beamer":
			self.spinFontSize.set_adjustment(self._adjBeamer)
		else:
			self.spinFontSize.set_adjustment(self._adjDefault)
			if documentClass in ["article", "report", "book", "letter"]:
				self.spinFontSize.set_value(10)
			else:
				self.spinFontSize.set_value(11)
	
	def getSource(self):
		""" Returns the source corresponding to the settings made """
		import string
		
		documentClass = self._availClasses[self.cmbClass.get_active()]
		
		mapping = {
				"class" : documentClass,
				"options" : "%spt,%s" % (self.spinFontSize.get_value_as_int(), self._availPaperSizes[self.cmbPaperSize.get_active()]),
				"inputenc" : "\\usepackage[%s]{inputenc}\n" % self._availEncodings[self.cmbEncoding.get_active()],
				"babel" : "\\usepackage[%s]{babel}\n" % self._availBabels[self.cmbBabel.get_active()],
				"title" : self.entryTitle.get_text(),
				"author" : self.entryAuthor.get_text(),
				"date" : self.entryDate.get_text()
		}
		
		if len(self._availThemes):
			mapping["theme"] = self._availThemes[self.cmbTheme.get_active()]
		
		if self.cmbPaperAlign.get_active():
			mapping["options"] += ",landscape"
		
		try:
			template = open("%s/templates/%s" % (const.PATH, documentClass)).read()
		except IOError:
			template = open(const.PATH + "/templates/default").read()
			
		source = string.Template(template).safe_substitute(mapping)
		return tuple(source.split("$_"))
	
	def _findAvailable(self, names, prefix="", suffix=""):
		""" Builds filenames from names, prefix and suffix and returns only the
			available of them """
		filenames = ["%s%s%s" % (prefix, name, suffix) for name in names]
		foundFilenames = os.popen("kpsewhich " + " ".join(filenames)).read().splitlines()
		foundNames = [os.path.splitext(os.path.basename(filename))[0][len(prefix):] for filename in foundFilenames]
		return foundNames
