# -*- coding: utf-8 -*-

# RenderImageDialog.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
import const

PNG_MODES = [
		("mono", _("Monochrome")),
		("gray", _("Grayscale")),
		("16", _("RGB (4 bit)")),
		("256", _("RGB (8 bit)")),
		("16m", _("RGB (24 bit)")),
		("alpha", _("RGBA (transparent)"))
]

AA_FACTORS = [0, 1, 2, 4, 8]

class RenderImageDialog(gtk.Dialog):
	def __init__(self):
		gtk.Dialog.__init__(self, _("Render to Image"), None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
				(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(10)
		
		box = gtk.VBox()
		box.set_border_width(5)
		box.set_spacing(10)
		
		## PNG
		self.radioPNG = gtk.RadioButton(None, "PNG")
		
		self.lblPNG = gtk.Label(_("Color Mode:"))
		self.lblPNG.set_alignment(0, .5)
		self.cmbPNG = gtk.combo_box_new_text()
		for pair in PNG_MODES:
			self.cmbPNG.append_text(pair[1])
		self.cmbPNG.set_active(4)
		
		hboxPNG = gtk.HBox()
		hboxPNG.set_spacing(5)
		hboxPNG.pack_start(self.lblPNG, False)
		hboxPNG.pack_start(self.cmbPNG, False)
		
		algPNG = gtk.Alignment(.5, .5, 1, 1)
		algPNG.set_padding(0, 0, 20, 0)
		algPNG.add(hboxPNG)
		
		## JPEG, GIF
		self.radioJPG = gtk.RadioButton(self.radioPNG, "JPEG")
		self.radioGIF = gtk.RadioButton(self.radioPNG, "GIF")
		
		vboxFormat = gtk.VBox()
		vboxFormat.set_spacing(2)
		vboxFormat.add(self.radioPNG)
		vboxFormat.add(algPNG)
		vboxFormat.add(self.radioJPG)
		vboxFormat.add(self.radioGIF)
		
		algFormat = gtk.Alignment(.5, .5, 1, 1)
		algFormat.set_padding(5, 0, 15, 0)
		algFormat.add(vboxFormat)
		
		frmFormat = gtk.Frame(_("<b>Image Format</b>"))
		frmFormat.get_label_widget().set_use_markup(True)
		frmFormat.set_shadow_type(gtk.SHADOW_NONE)
		frmFormat.add(algFormat)
		
		box.add(frmFormat)
		
		# Quality
		lblAA = gtk.Label(_("Antialiasing Factor:"))
		lblAA.set_alignment(0, .5)
		
		self.cmbAA = gtk.combo_box_new_text()
		for factor in AA_FACTORS:
			self.cmbAA.append_text(str(factor))
		self.cmbAA.set_active(3)
		
		lblRes = gtk.Label(_("Resolution:"))
		lblRes.set_alignment(0, .5)
		lblResU = gtk.Label("dpi")
		lblResU.set_alignment(0, .5)
		
		self.spinRes = gtk.SpinButton(gtk.Adjustment(120, 1, 999, 1), 1)
		
		tblQuality = gtk.Table()
		tblQuality.set_row_spacings(5)
		tblQuality.set_col_spacings(5)
		tblQuality.attach(lblAA, 0, 1, 0, 1, gtk.FILL, False)
		tblQuality.attach(self.cmbAA, 1, 2, 0, 1, gtk.FILL, False)
		tblQuality.attach(lblRes, 0, 1, 1, 2, gtk.FILL, False)
		tblQuality.attach(self.spinRes, 1, 2, 1, 2, gtk.FILL, False)
		tblQuality.attach(lblResU, 2, 3, 1, 2, gtk.FILL, False)
		
		algQuality = gtk.Alignment(.5, .5, 1, 1)
		algQuality.set_padding(5, 0, 15, 0)
		algQuality.add(tblQuality)
		
		frmQuality = gtk.Frame(_("<b>Quality</b>"))
		frmQuality.get_label_widget().set_use_markup(True)
		frmQuality.set_shadow_type(gtk.SHADOW_NONE)
		frmQuality.add(algQuality)
		
		box.add(frmQuality)
		
		## Size
		self.radioPage = gtk.RadioButton(None, _("Render the whole page(s)"))
		self.radioBox = gtk.RadioButton(self.radioPage, _("Attempt to render a tight bounding box"))
		
		lblBox = gtk.Label()
		lblBox.set_markup(_("<small><u>Note:</u> This only works on one-page files and requires the <tt>\pagestyle{empty}</tt> command.</small>"))
		lblBox.set_line_wrap(True)
		
		algBox = gtk.Alignment(.5, .5, 1, 1)
		algBox.set_padding(0, 0, 20, 0)
		algBox.add(lblBox)
		
		vboxSize = gtk.VBox()
		vboxSize.set_spacing(2)
		vboxSize.add(self.radioPage)
		vboxSize.add(self.radioBox)
		vboxSize.add(algBox)
		
		algSize = gtk.Alignment(.5, .5, 1, 1)
		algSize.set_padding(5, 0, 15, 0)
		algSize.add(vboxSize)
		
		frmSize = gtk.Frame(_("<b>Output Size</b>"))
		frmSize.get_label_widget().set_use_markup(True)
		frmSize.set_shadow_type(gtk.SHADOW_NONE)
		frmSize.add(algSize)
		
		box.add(frmSize)
		self.vbox.add(box)
		
		self.connect("response", self.__onResponse)
		self.radioPNG.connect("toggled", self.__onFormatToggled)
		self.radioJPG.connect("toggled", self.__onFormatToggled)
		self.radioGIF.connect("toggled", self.__onFormatToggled)
		self.set_default_response(gtk.RESPONSE_OK)
	
	def run(self):
		self.show_all()
		return gtk.Dialog.run(self)
	
	def __onFormatToggled(self, radio):
		self.lblPNG.set_sensitive(self.radioPNG.get_active())
		self.cmbPNG.set_sensitive(self.radioPNG.get_active())
	
	def __onResponse(self, widget, response):
		if response == gtk.RESPONSE_OK:
			if self.radioPNG.get_active():
				self.format = const.PNG
			elif self.radioJPG.get_active():
				self.format = const.JPG
			elif self.radioGIF.get_active():
				self.format = const.GIF
			self.pngMode = PNG_MODES[self.cmbPNG.get_active()][0]
			self.aaFactor = AA_FACTORS[self.cmbAA.get_active()]
			self.boundingBox = self.radioBox.get_active()
			self.resolution = self.spinRes.get_value()
		
