# -*- coding: utf-8 -*-

# Structure.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
import const

class Structure(gtk.VPaned):
	pbLabel = gtk.gdk.pixbuf_new_from_file(const.PATH + "/pixmaps/label.png")
	pbBibitem = gtk.gdk.pixbuf_new_from_file(const.PATH + "/pixmaps/bibitem.png")
	
	def __init__(self, instance):
		self._instance = instance
		gtk.VPaned.__init__(self)
		
		self.viewStructure = gtk.TreeView()
		self.viewStructure.set_headers_visible(False)
		self.viewStructure.append_column(gtk.TreeViewColumn(None, gtk.CellRendererText(), markup=0))
		self.viewStructure.connect("row-activated", self.onRowSelected)
		
		scrStructure = gtk.ScrolledWindow()
		scrStructure.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);
		scrStructure.set_shadow_type(gtk.SHADOW_IN)
		scrStructure.add(self.viewStructure)
		
		self.add1(scrStructure)
		
		self.viewLabels = gtk.TreeView()
		self.viewLabels.set_headers_visible(False)
		cellPixbuf = gtk.CellRendererPixbuf()
		self.viewLabels.insert_column_with_attributes(-1, "", cellPixbuf)
		self.viewLabels.get_column(0).set_cell_data_func(cellPixbuf, self.rowPixbuf)
		self.viewLabels.insert_column_with_attributes(-1, "", gtk.CellRendererText(), text=0)
		self.viewLabels.connect("row-activated", self.onLabelSelected)
		
		scrLabels = gtk.ScrolledWindow()
		scrLabels.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);
		scrLabels.set_shadow_type(gtk.SHADOW_IN)
		scrLabels.add(self.viewLabels)
		
		self.add2(scrLabels)
		
		self.show_all()
	
	def rowPixbuf(self, column, cell, model, iter):
		""" Cell Data Function """
		t = model.get_value(iter, 1)
		if t == const.LABEL:
			pixbuf = self.pbLabel
		elif t == const.BIBITEM:
			pixbuf = self.pbBibitem
		cell.set_property('pixbuf', pixbuf)
			
	def onRowSelected(self, treeview, path, column):
		""" Scrolls the document to the selected row """
		treestore = treeview.get_model()
		string = treestore.get_value(treestore.get_iter(path), 1)
		self._instance.get_document().findString(string)
	
	def onLabelSelected(self, treeview, path, column):
		liststore, listiter = treeview.get_selection().get_selected()
		string = liststore.get_value(listiter, 2)
		self._instance.get_document().findString(string)
