# -*- coding: utf-8 -*-

# Symbols.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
import const

SYMBOLS = {
	"operators" : ("approx","ast","bullet","cap","cdot","circ","cong","cup","div","equiv","eth","exists","forall","geq","gg","in","land","leq","ll","lor","mid","nabla","neg","neq","nexists","ni","parallel","partial","pm","propto","setminus","sim","simeq","star","subset","subseteq","supset","supseteq","times","uplus","varnothing"),
	"arrows" 	: ("Leftarrow","Leftrightarrow","Longleftarrow","Longleftrightarrow","Longrightarrow","Rightarrow","hookleftarrow","hookrightarrow","leadsto","leftarrow","leftharpoondown","leftharpoonup","leftrightarrow","leftrightarrows","leftrightharpoons","longleftarrow","longleftrightarrow","longmapsto","longrightarrow","mapsto","rightarrow","rightharpoondown","rightharpoonup","rightleftarrows","rightleftharpoons"),
	"limiters" 	: ("lbrace","rbrace","langle","rangle","lfloor","rfloor","lceil","rceil","vert","Vert"),
	"greek" 	: ("alpha","beta","gamma","delta","epsilon","varepsilon","zeta","eta","theta","vartheta","iota","kappa","lambda","mu","nu","xi","pi","varpi","rho","varrho","sigma","varsigma","tau","upsilon","phi","varphi","chi","psi","omega","Gamma","Delta","Theta","Lambda","Xi","Pi","Sigma","Upsilon","Phi","Psi","Omega"),
	"special" 	: ("ldots","cdots","vdots","ddots","Im","Re","complement","hbar","infty","S","textcopyright","textregistered","texttrademark","P","dagger","wp","aleph", "angle", "prime", "ell")
}

class Symbols(gtk.ScrolledWindow):
	def __init__(self, instance):
		gtk.ScrolledWindow.__init__(self)
		self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		
		self._instance = instance
		
		self.vbox = gtk.VBox()
		self.add_with_viewport(self.vbox)
		
		for page in SYMBOLS.keys():
			model = gtk.ListStore(gtk.gdk.Pixbuf)
			
			for symbol in SYMBOLS[page]:
				model.append([gtk.gdk.pixbuf_new_from_file("%s/pixmaps/%s/%s.png" % (const.PATH, page, symbol))])
			
			view = gtk.IconView(model)
			view.set_pixbuf_column(0)
			view.set_selection_mode(gtk.SELECTION_SINGLE)
			view.connect("selection-changed", self.__onSymbolSelected, page)
			view.set_item_width(-1)			# use largest item width
			view.set_spacing(0)
			view.set_column_spacing(0)
			view.set_row_spacing(0)
			view.set_columns(-1)			# show as much as possible
			view.set_text_column(-1)
			
			self.vbox.pack_start(view, True)

		self.show_all()
	
	def __onSymbolSelected(self, iconView, symbolPage):
		try:
			index = iconView.get_selected_items()[0][0]
		except IndexError:		# must be caught after unselect_all()
			return
		self._instance.get_document().insert("\\" + SYMBOLS[symbolPage][index])
		iconView.unselect_all()
	
