# -*- coding: utf-8 -*-

# TableDialog.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
import const

class TableDialog(gtk.Dialog):
	source, package = None, None
	
	def __init__(self):
		gtk.Dialog.__init__(self, _("Insert Tabular Object"), None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
				(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(10)
		
		## contents
		self.fileDialog = gtk.FileChooserDialog(action=gtk.FILE_CHOOSER_ACTION_OPEN,
				buttons=(gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
		self.fileDialog.set_local_only(True)
		self.fileDialog.connect("response", self.__onFileResponse)
		#self.fileDialog.connect("file-activated", self.__onFileActivated)
		self.fileButton = gtk.FileChooserButton(self.fileDialog)
		
		algContents = gtk.Alignment(.5, .5, 1, 1)
		algContents.set_padding(5, 0, 15, 0)
		algContents.add(self.fileButton)
		
		frmContents = gtk.Frame(_("<b>Import Data</b>"))
		frmContents.get_label_widget().set_use_markup(True)
		frmContents.set_shadow_type(gtk.SHADOW_NONE)
		frmContents.add(algContents)
		
		## general
		self.spinRows = gtk.SpinButton(gtk.Adjustment(1, 1, 1000, 1), 1)
		self.spinRows.set_width_chars(7)
		self.spinRows.set_activates_default(True)
		algRows = gtk.Alignment()
		algRows.add(self.spinRows)
		
		self.spinCols = gtk.SpinButton(gtk.Adjustment(2, 1, 1000, 1), 1)
		self.spinCols.set_width_chars(7)
		self.spinCols.set_activates_default(True)
		self.spinCols.connect("value-changed", self.__onColumnsChanged)
		algColumns = gtk.Alignment()
		algColumns.add(self.spinCols)
		
		lblRows = gtk.Label(_("_Rows:"))
		lblRows.set_mnemonic_widget(self.spinRows)
		lblRows.set_use_underline(True)
		lblRows.set_alignment(0, .5)
		
		lblColumns = gtk.Label(_("_Columns:"))
		lblColumns.set_mnemonic_widget(self.spinCols)
		lblColumns.set_use_underline(True)
		lblColumns.set_alignment(0, .5)
		
		self.cmbLayout = gtk.combo_box_entry_new_text()
		self.cmbLayout.append_text("l")
		
		lblLayout = gtk.Label(_("Column _Layout:"))
		lblLayout.set_mnemonic_widget(self.cmbLayout)
		lblLayout.set_use_underline(True)
		lblLayout.set_alignment(0, .5)
		
		tblGeneral = gtk.Table()
		tblGeneral.set_row_spacings(5)
		tblGeneral.set_col_spacings(5)
		tblGeneral.attach(lblRows, 0, 1, 0, 1, gtk.FILL, False)
		tblGeneral.attach(algRows, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, False)
		tblGeneral.attach(lblColumns, 0, 1, 1, 2, gtk.FILL, False)
		tblGeneral.attach(algColumns, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, False)
		tblGeneral.attach(lblLayout, 0, 1, 2, 3, gtk.FILL, False)
		tblGeneral.attach(self.cmbLayout, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, False)
		
		algGeneral = gtk.Alignment(.5, .5, 1, 1)
		algGeneral.set_padding(5, 0, 15, 0)
		algGeneral.add(tblGeneral)
		
		frmGeneral = gtk.Frame(_("<b>General</b>"))
		frmGeneral.get_label_widget().set_use_markup(True)
		frmGeneral.set_shadow_type(gtk.SHADOW_NONE)
		frmGeneral.add(algGeneral)
		
		## table
		self.radioGeneral = gtk.RadioButton(None, _("_General (Table)"))
		self.radioGeneral.connect("toggled", self.__onRadioToggled)
		
		## matrix
		self.radioMath = gtk.RadioButton(self.radioGeneral, _("_Mathematical (Matrix)"))
		self.radioMath.connect("toggled", self.__onRadioToggled)
		
		## delimiters
		modelDelims = gtk.ListStore(gtk.gdk.Pixbuf, str)
		modelDelims.append([None, "None"])
		for delim in [("parantheses", _("Parantheses")), ("brackets", _("Brackets")), ("braces", _("Braces")), ("vbars", _("Vertical Bars")), ("dvbars", _("Double Vertical Bars"))]:
			pixbuf = gtk.gdk.pixbuf_new_from_file("%s/pixmaps/%s.png" % (const.PATH, delim[0]))
			modelDelims.append([pixbuf, delim[1]])
		
		self.cmbDelims = gtk.ComboBox(modelDelims)
		cellPixbuf = gtk.CellRendererPixbuf()
  		self.cmbDelims.pack_start(cellPixbuf, False)
  		self.cmbDelims.add_attribute(cellPixbuf, 'pixbuf', 0)
		cellText = gtk.CellRendererText()
  		self.cmbDelims.pack_start(cellText, False)
  		self.cmbDelims.add_attribute(cellText, 'text', 1)
  		self.cmbDelims.set_active(0)
		
		self.lblDelims = gtk.Label(_("_Delimiters:"))
		self.lblDelims.set_mnemonic_widget(self.cmbDelims)
		self.lblDelims.set_use_underline(True)
		self.lblDelims.set_alignment(0, .5)
		
		self.checkAMS = gtk.CheckButton("Use _AMS environment")
		
		## pack
		tblEnviron = gtk.Table()
		tblEnviron.set_row_spacings(5)
		tblEnviron.set_col_spacings(5)
		tblEnviron.attach(self.radioGeneral, 0, 3, 0, 1)
		tblEnviron.attach(self.radioMath, 0, 3, 1, 2)
		tblEnviron.attach(self.lblDelims, 1, 2, 2, 3, gtk.FILL, False)
		tblEnviron.attach(self.cmbDelims, 2, 3, 2, 3, gtk.FILL|gtk.EXPAND, False)
		tblEnviron.attach(self.checkAMS, 1, 3, 3, 4, gtk.FILL, False)
		tblEnviron.set_col_spacing(0, 25)
		
		algEnviron = gtk.Alignment(.5, .5, 1, 1)
		algEnviron.set_padding(5, 0, 15, 0)
		algEnviron.add(tblEnviron)
		
		frmEnviron = gtk.Frame(_("<b>Environment</b>"))
		frmEnviron.get_label_widget().set_use_markup(True)
		frmEnviron.set_shadow_type(gtk.SHADOW_NONE)
		frmEnviron.add(algEnviron)
		
		vbox = gtk.VBox()
		vbox.set_border_width(5)
		vbox.set_spacing(10)
		vbox.pack_start(frmContents, False)
		vbox.pack_start(frmGeneral, False)
		vbox.pack_start(frmEnviron, False)
		
		self.vbox.add(vbox)
		
		self.radioGeneral.set_active(True)
		self.set_default_response(gtk.RESPONSE_OK)
		self.connect("response", self.__onResponse)
		
		self.__onRadioToggled(self.radioGeneral)
		self.__onColumnsChanged(self.spinCols)
		self.cmbLayout.set_active(0)
	
	def __onFileResponse(self, dialog, response):
		if response == gtk.RESPONSE_ACCEPT:
			self.imValues = [row.split() for row in open(dialog.get_filename()).readlines() if row[0] != '#' and len(row.split())]
			self.imRows = len(self.imValues)
			if self.imRows:
				self.imCols = len(self.imValues[0])
			else:
				self.imCols = 0
				
			if self.imRows and self.imCols:
				self.spinRows.set_value(self.imRows)
				self.spinCols.set_value(self.imCols)
				self.spinRows.set_sensitive(False)
				self.spinCols.set_sensitive(False)
	
	def __onRadioToggled(self, radioButton):
		state = self.radioMath.get_active()
		self.lblDelims.set_sensitive(state)
		self.cmbDelims.set_sensitive(state)
		self.checkAMS.set_sensitive(state)
	
	def __onColumnsChanged(self, spinButton):
		""" suggest layouts that fit the number of columns """
		cols = spinButton.get_value_as_int()
		self.cmbLayout.get_model().clear()
		for layout in ["l" * cols, "|".join("l" * cols), "@{\\ \\vline\\ }".join("l" * cols)]:
			self.cmbLayout.append_text(layout)	
	
	def __buildSource(self):
		if self.imValues:
			## create a grid from the imported values
			rows = ["\t" + " & ".join(row) for row in self.imValues]
			grid = " \\\\\n".join(rows)
		else:
			## create a grid of placeholders
			r = "\t" + " & ".join("x" * self.spinCols.get_value_as_int())
			grid = " \\\\\n".join([r] * self.spinRows.get_value_as_int())
		
		## wrap it into an environment
		if self.radioGeneral.get_active():
			self.source = "\\begin{tabular}{%s}\n%s\n\\end{tabular}" % (self.cmbLayout.get_active_text(), grid)
		else:
			delimIndex = self.cmbDelims.get_active()
			if self.checkAMS.get_active():
				environment = ["matrix", "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix"][delimIndex]
				self.source = "\\begin{%s}\n%s\n\\end{%s}" % (environment, grid, environment)
				self.package = "amsmath"
			else:
				lDelim = ["", r"\left(", r"\left[", r"\left\lbrace", r"\left\vert", r"\left\Vert"][delimIndex]
				rDelim = ["", r"\right(", r"\right[", r"\right\rbrace", r"\right\vert", r"\right\Vert"][delimIndex]
				self.source = "%s\\begin{array}{%s}\n%s\n\\end{array}%s" % (lDelim, self.cmbLayout.get_active_text(), grid, rDelim)
	
	def __onResponse(self, dialog, response):
		if response == gtk.RESPONSE_OK:
			self.__buildSource()
	
	def run(self, environment):
		## reset ui
		self.fileDialog.unselect_all()
		self.spinRows.set_sensitive(True)
		self.spinCols.set_sensitive(True)
		self.imValues = None
		self.radioGeneral.set_active(environment == const.GENERAL)
		self.radioMath.set_active(environment == const.MATH)
		
		self.show_all()
		return gtk.Dialog.run(self)
