# -*- coding: utf-8 -*-

# __init__.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gedit, gtk, os.path
from config import Preferences
from DocumentWrapper import *
from Structure import Structure
from Symbols import Symbols
from macros import Macros
#from nmacros import Macros
from completion import CompletionPopup
import const

DBUS_BUS_NAME = 'org.gedit.LaTeXPlugin'
DBUS_OBJECT_PATH = '/org/gedit/DBUSInterface'

CUSTOM_ICONS = {
		"/LaTeXToolbar/QuickstartTool" : "/pixmaps/quickstart.png",
		
		"/LaTeXToolbar/BoldTool" : "/pixmaps/bf.png",
		"/LaTeXToolbar/ItalicTool" : "/pixmaps/it.png",
		"/LaTeXToolbar/UnderlineTool" : "/pixmaps/underline.png",
		"/LaTeXToolbar/SmallCapsTool" : "/pixmaps/sc.png",
		
		"/LaTeXToolbar/CenterTool" : "/pixmaps/center.png",
		
		"/LaTeXToolbar/ItemizeTool" : "/pixmaps/itemize.png",
		"/LaTeXToolbar/EnumerateTool" : "/pixmaps/enumerate.png",
		"/LaTeXToolbar/DescriptionTool" : "/pixmaps/description.png",
		"/LaTeXToolbar/ItemTool" : "/pixmaps/item.png",
		
		"/LaTeXToolbar/SubscriptTool" : "/pixmaps/sub.png",
		"/LaTeXToolbar/SuperscriptTool" : "/pixmaps/super.png",
		"/LaTeXToolbar/FractionTool" : "/pixmaps/frac.png",
		"/LaTeXToolbar/GraphicsTool" : "/pixmaps/graphics.png",
		"/LaTeXToolbar/TableTool" : "/pixmaps/table.png",
		
		"/TypefacePopup/TypefaceRmMenu" : "/pixmaps/rm.png",
		"/TypefacePopup/TypefaceSfMenu" : "/pixmaps/sf.png",
		"/TypefacePopup/TypefaceTtMenu" : "/pixmaps/tt.png",
		"/TypefacePopup/TypefaceBbMenu" : "/pixmaps/bb.png",
		"/TypefacePopup/TypefaceCalMenu" : "/pixmaps/cal.png",
		"/TypefacePopup/TypefaceFrakMenu" : "/pixmaps/frak.png",
		
		"/StructurePopup/StructurePartMenu" : "/pixmaps/part.png",
		"/StructurePopup/StructureChapterMenu" : "/pixmaps/chapter.png",
		"/StructurePopup/StructureSectionMenu" : "/pixmaps/section.png",
		"/StructurePopup/StructureSubsectionMenu" : "/pixmaps/subsection.png",
		"/StructurePopup/StructureSubsubsectionMenu" : "/pixmaps/subsubsection.png",
		"/StructurePopup/StructureParagraphMenu" : "/pixmaps/paragraph.png",
		"/StructurePopup/StructureSubparagraphMenu" : "/pixmaps/paragraph.png",
		
		"/MathMenu/DisplayMathMenu" : "/pixmaps/displaymath.png",
		"/MathMenu/EquationMenu" : "/pixmaps/equation.png",
		"/MathMenu/UnnumberedEqnArrayMenu" : "/pixmaps/uneqnarray.png",
		"/MathMenu/NumberedEqnArrayMenu" : "/pixmaps/eqnarray.png",
		
		"/AccentPopup/AccentDotMenu" : "/pixmaps/dot.png",
		"/AccentPopup/AccentDdotMenu" : "/pixmaps/ddot.png",
		"/AccentPopup/AccentBarMenu" : "/pixmaps/bar.png",
		"/AccentPopup/AccentTildeMenu" : "/pixmaps/tilde.png",
		"/AccentPopup/AccentVecMenu" : "/pixmaps/vec.png",
		"/AccentPopup/AccentHatMenu" : "/pixmaps/hat.png",
		"/AccentPopup/AccentAcuteMenu" : "/pixmaps/acute.png",
		"/AccentPopup/AccentGraveMenu" : "/pixmaps/grave.png",
		"/AccentPopup/AccentBreveMenu" : "/pixmaps/breve.png",
		"/AccentPopup/AccentCheckMenu" : "/pixmaps/check.png",
		
		"/CompilePopup/CompilePDFMenu" : "/pixmaps/compile-pdf.png",
		"/CompilePopup/CompileDVIMenu" : "/pixmaps/compile-dvi.png",
		"/CompilePopup/CompilePSMenu" : "/pixmaps/compile-ps.png",
		"/CompilePopup/CompileImageMenu" : "/pixmaps/compile-image.png"
}

class C:
	"""
	Encapsulates a LaTeX macro or command
	"""
	def __init__(self, components, package=None):
		self._components = components
		self._package = package
	def __getitem__(self, index):
		return self._components[index]
	def __len__(self):
		return len(self._components)
	def get_package(self):
		return self._package

ACTIONS_TO_MACROS = {
		"Bold" : (C((r"\textbf{", "}")), C((r"\mathbf{", "}"))),
		"Italic" : (C((r"\textit{", "}")), C((r"\mathit{", "}"))),
		"Underline" : (C((r"\underline{", "}")), C((r"\underline{", "}"))),
		"SmallCaps" : (C((r"\textsc{", "}")), None),
		
		"Center" : (C(("\\begin{center}\n\t", "\n\\end{center}")), None),
		
		"Itemize" : (C(("\\begin{itemize}\n\t\\item", "\n\\end{itemize}")), None),
		"Enumerate" : (C(("\\begin{enumerate}\n\t\\item", "\n\\end{enumerate}")), None),
		"Description" : (C(("\\begin{description}\n\t\\item[", "]\n\\end{description}")), None),
		"Item" : (C(("\\item ")), None),
		
		"DisplayMath" : (C(("\[ ", " \]")), None),
		"Equation" : (C(("\\begin{equation}\n\t", "\n\\end{equation}")), None),
		"UnnumberedEqnArray" : (C(("\\begin{align*}\n\t", "\n\\end{align*}"), "amsmath"), None),
		"NumberedEqnArray" : (C(("\\begin{align}\n\t", "\n\\end{align}"), "amsmath"), None),
		
		"TypefaceRm" : (C(("\\textrm{", "}")), C(("\\mathrm{", "}"))),
		"TypefaceSf" : (C(("\\textsf{", "}")), C(("\\mathsf{", "}"))),
		"TypefaceTt" : (C(("\\texttt{", "}")), C(("\\mathtt{", "}"))),
		"TypefaceBb" : (None, C(("\\mathbb{", "}"), "amssymb")),
		"TypefaceCal" : (None, C(("\\mathcal{", "}"))),
		"TypefaceFrak" : (None, C(("\\mathfrak{", "}"), "amssymb")),
		
		"StructurePart" : (C(("\\part{", "}")), None),
		"StructureChapter" : (C(("\\chapter{", "}")), None),
		"StructureSection" : (C(("\\section{", "}")), None),
		"StructureSubsection" : (C(("\\subsection{", "}")), None),
		"StructureSubsubsection" : (C(("\\subsubsection{", "}")), None),
		"StructureParagraph" : (C(("\\paragraph{", "}")), None),
		"StructureSubparagraph" : (C(("\\subparagraph{", "}")), None),
		
		"Subscript" : (None, C((r"_{", "}"))),
		"Superscript" : (None, C((r"^{", "}"))),
		"Fraction" : (None, C((r"\frac{", "}{}"))),
		
		"AccentDot" : (C(("\\.{", "}")), C(("\\dot{", "}"))),
		"AccentDdot" : (C(('\\"{', "}")), C(("\\ddot{", "}"))),
		"AccentBar" : (None, C(("\\bar{", "}"))),
		"AccentTilde" : (C(("\\~{", "}")), C(("\\tilde{", "}"))),
		"AccentVec" : (None, C(("\\vec{", "}"))),
		"AccentHat" : (C(("\\^{", "}")), C(("\\hat{", "}"))),
		"AccentAcute" : (C(("\\'{", "}")), C(("\\acute{", "}"))),
		"AccentGrave" : (C(("\\`{", "}")), C(("\\grave{", "}"))),
		"AccentBreve" : (None, C(("\\breve{", "}"))),
		"AccentCheck" : (None, C(("\\check{", "}"))),
}

# widgets that are only defined is special LaTeX environments
SMART_WIDGETS = {
		"/LaTeXToolbar/SmallCapsTool" : const.GENERAL,
		"/LaTeXToolbar/ItemizeTool" : const.GENERAL,
		"/LaTeXToolbar/EnumerateTool" : const.GENERAL,
		"/LaTeXToolbar/DescriptionTool" : const.GENERAL,
		"/LaTeXToolbar/ItemTool" : const.GENERAL,
		"/LaTeXToolbar/SubscriptTool" : const.MATH,
		"/LaTeXToolbar/SuperscriptTool" : const.MATH,
		"/LaTeXToolbar/FractionTool" : const.MATH,
		"/AccentPopup/AccentBarMenu" : const.MATH,
		"/AccentPopup/AccentVecMenu" : const.MATH,
		"/AccentPopup/AccentBreveMenu" : const.MATH,
		"/AccentPopup/AccentCheckMenu" : const.MATH,
		"/TypefacePopup/TypefaceBbMenu" : const.MATH,
		"/TypefacePopup/TypefaceCalMenu" : const.MATH,
		"/TypefacePopup/TypefaceFrakMenu" : const.MATH,
		"/LaTeXToolbar/GraphicsTool" : const.GENERAL
}

class LaTeXPluginInstance:
	_document_wrappers = {}		# maps the GeditViews to DocumentWrappers
	_last_wrapper = None		# the last DocumentWrapper activated
	_wrapper_connections = {}	# maps signals to handler ids for reconnecting
	
	_quickstartDialog = None
	_graphicsDialog = None
	_tableDialog = None
	_backend = None
	_backendDialog = None
	_renderImageDialog = None
	
	def __init__(self, plugin, window):
		self._plugin = plugin
		self._window = window
		
		self._prefs = Preferences()
		self._manager = window.get_ui_manager()
		self._saveAction = self._manager.get_action("/MenuBar/FileMenu/FileSaveMenu")
		self._structure = Structure(self)
		self._structure.set_position(self._prefs.setdefault("/structure-paned-position", -1))
		self._symbols = Symbols(self)
		self._macros = Macros(self)
		self._macros.set_position(self._prefs.setdefault("/macros-paned-position", -1))
		
		self._clipboard = gtk.clipboard_get("PRIMARY")
		
		self._window.get_side_panel().add_item(self._structure, _("Document Structure"), gtk.image_new_from_stock(gtk.STOCK_INDEX, gtk.ICON_SIZE_MENU))
		self._window.get_side_panel().add_item(self._symbols, _("Common Symbols"), gtk.image_new_from_stock(gtk.STOCK_SELECT_FONT, gtk.ICON_SIZE_MENU))
		self._window.get_side_panel().add_item(self._macros, _("Macros and New Commands"), gtk.image_new_from_stock(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_MENU))
		
		self._builtinCompletionTargets = open(const.PATH + "/completion/builtin").read().splitlines()
		self._completionPopup = CompletionPopup()
		
		self._insert_ui()
		
		window.connect("destroy", lambda a: self._cleanup())
		
		self.dviSearchEnabled = self._prefs.setdefault("/dvi-search-enabled", False)
		if self.dviSearchEnabled:
			# load DBUS modules and create a service
			import dbus.service
			if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
				import dbus.glib
				
			class DBUSInterface(dbus.service.Object):
				"""
				The DBUSInterface object is listening for DVI inverse search commands and
				scrolls the document to the appropriate line
				"""
				def __init__(self, instance, bus_name, object_path=DBUS_OBJECT_PATH):
					dbus.service.Object.__init__(self, bus_name, object_path)
					self._instance = instance

				# This is for dbus inverse search
				@dbus.service.method('org.gedit.LaTeXPluginIFace')
				def inverse(self, filename, line):
					print "Got inverse DVI search: " + filename + " " + str(line)
					# TODO: switch to the appropriate tab if the file is not active
					document = self._instance.get_document()
					# FIXME: document.tempFilename is not used anymore
					# Backend now handles temp files on its own
					if os.path.samefile(document.getFilename(), filename):
						document.scrollToLine(line - 1)		# tex counts from 1
					else:	
						print "Error: file not loaded or active"

				# adding this makes things work---don't know why!
				@dbus.service.method('org.gedit.LaTeXPluginIFace')
				def dummy(self):
					pass
		
			# DBUS interface
			bus_name = dbus.service.BusName(DBUS_BUS_NAME, bus=dbus.SessionBus())
			self._dbus = DBUSInterface(self, bus_name)
	
	def get_document(self):
		""" returns the DocumentWrapper of currently selected GeditView or None if
			there are no tabs """
		active_view = self._window.get_active_view()
		if active_view:
			try:
				document = self._document_wrappers[active_view]
			except KeyError:
				return None
			return document
		return None
	
	def _insert_ui(self):
		""" merges the plugin's UI """
		self._action_group = gtk.ActionGroup("LaTeXPluginActions")
		self._action_group.add_actions([
				("Comment", None, _("Comment"), "<Ctrl><Alt>C", None, lambda a: self.get_document().comment_selection()),
				("Uncomment", None, _("Uncomment"), None, None, lambda a: self.get_document().comment_selection(True)),
				
				("RefreshStructure", gtk.STOCK_REFRESH, _("Refresh Document Structure"), "F5", _("Parse the document again and refresh the structure view"), lambda a: self.get_document().update()),
				("FindInDVI", gtk.STOCK_FIND, _("Find selection in DVI"), "<Shift><Ctrl>F", _("Finds the current position in the DVI document"), lambda a: self._performDVISearch()),
				("SearchPrimary", gtk.STOCK_FIND, _("Find primary selection"), "<Shift><Ctrl>Q", _("Finds the primary selection in the current document"), lambda a: self._performPrimarySearch()),
				("Compile", gtk.STOCK_CONVERT, _("Compile LaTeX"), None, None, None),
				("CompilePDF", None, "PDF", "F6", _("Compiles the document to PDF"), lambda a: self._compileDocument(const.PDF)),
				("CompileDVI", None, "DVI", "<Ctrl>F6", _("Compiles the document to DVI"), lambda a: self._compileDocument(const.DVI)),
				("CompilePS", None, "PostScript", "<Shift><Ctrl>F6", _("Compiles the document to PostScript"), lambda a: self._compileDocument(const.PS)),
				("CompileImage", None, _("Image") + "...", None, _("Renders the document to an image"), lambda a: self._renderToImage()),
				
				("Quickstart", gtk.STOCK_MISSING_IMAGE, _("Quickstart"), None, _("Create a new LaTeX document"), lambda a: self._runQuickstartDialog()),
				("Bold", gtk.STOCK_MISSING_IMAGE, _("Bold"), None, _("Bold"), self._insert_macro),
				("Italic", gtk.STOCK_MISSING_IMAGE, _("Italic"), None, _("Italic"), self._insert_macro),
				("Underline", gtk.STOCK_MISSING_IMAGE, _("Underline"), None, _("Underline"), self._insert_macro),
				("SmallCaps", gtk.STOCK_MISSING_IMAGE, _("Small Caps"), None, _("Small Capitals"), self._insert_macro),
				
				("Center", gtk.STOCK_JUSTIFY_CENTER, _("Center"), None, _("Center"), self._insert_macro),
				
				("Itemize", gtk.STOCK_MISSING_IMAGE, _("List"), None, _("Insert a list"), self._insert_macro),
				("Enumerate", gtk.STOCK_MISSING_IMAGE, _("Enumeration"), None, _("Insert an enumeration"), self._insert_macro),
				("Description", gtk.STOCK_MISSING_IMAGE, _("Descriptions"), None, _("Insert a description list"), self._insert_macro),
				("Item", gtk.STOCK_MISSING_IMAGE, _("Item"), None, _("Insert an item into a list or enumeration"), self._insert_macro),
				
				("Graphics", gtk.STOCK_MISSING_IMAGE, _("Graphics"), None, _("Include a graphics"), lambda a: self._insertGraphics()),
				("Table", gtk.STOCK_MISSING_IMAGE, _("Table"), None, _("Insert a table or matrix"), lambda a: self._insertTable()),
				("Subscript", gtk.STOCK_MISSING_IMAGE, _("Subscript"), None, _("Subscript"), self._insert_macro),
				("Superscript", gtk.STOCK_MISSING_IMAGE, _("Superscript"), None, _("Superscript"), self._insert_macro),
				("Fraction", gtk.STOCK_MISSING_IMAGE, _("Fraction"), None, _("Fraction"), self._insert_macro),
				
				("TypefaceRm", None, _("Roman"), None, None, self._insert_macro),
				("TypefaceSf", None, _("Sans serif"), None, None, self._insert_macro),
				("TypefaceTt", None, _("Monospace"), None, None, self._insert_macro),
				("TypefaceBb", None, _("Blackboard Bold (Set)"), None, None, self._insert_macro),
				("TypefaceCal", None, _("Caligraphy"), None, None, self._insert_macro),
				("TypefaceFrak", None, _("Fraktur"), None, None, self._insert_macro),
				
				("StructurePart", None, _("Part"), None, None, self._insert_macro),
				("StructureChapter", None, _("Chapter"), None, None, self._insert_macro),
				("StructureSection", None, _("Section"), None, None, self._insert_macro),
				("StructureSubsection", None, _("Subsection"), None, None, self._insert_macro),
				("StructureSubsubsection", None, _("Sub-subsection"), None, None, self._insert_macro),
				("StructureParagraph", None, _("Paragraph"), None, None, self._insert_macro),
				("StructureSubparagraph", None, _("Subparagraph"), None, None, self._insert_macro),
				
				("DisplayMath", None, _("Centered Formula"), None, None, self._insert_macro),
				("Equation", None, _("Numbered Equation"), None, None, self._insert_macro),
				("UnnumberedEqnArray", None, _("Array of Equations"), None, None, self._insert_macro),
				("NumberedEqnArray", None, _("Array of Numbered Equations"), None, None, self._insert_macro),
				
				("AccentDot", None, _("Dot"), None, None, self._insert_macro),
				("AccentDdot", None, _("Double dot"), None, None, self._insert_macro),
				("AccentBar", None, _("Bar"), None, None, self._insert_macro),
				("AccentTilde", None, _("Tilde"), None, None, self._insert_macro),
				("AccentVec", None, _("Vector"), None, None, self._insert_macro),
				("AccentHat", None, _("Hat"), None, None, self._insert_macro),
				("AccentAcute", None, _("Acute"), None, None, self._insert_macro),
				("AccentGrave", None, _("Grave"), None, None, self._insert_macro),
				("AccentBreve", None, _("Breve"), None, None, self._insert_macro),
				("AccentCheck", None, _("Hacek"), None, None, self._insert_macro),
				
				("NewcommandInsert", gtk.STOCK_GO_FORWARD, _("Paste Command"), None, None, self._macros.onNewcommandContextInsert),
				("NewcommandJump", gtk.STOCK_JUMP_TO, _("Jump to Definition"), None, None, self._macros.onNewcommandContextJump),
				("NewcommandEdit", gtk.STOCK_EDIT, _("Edit..."), None, None, self._macros.onNewcommandContextEdit),
				("NewcommandDelete", gtk.STOCK_DELETE, _("Delete"), None, None, self._macros.onNewcommandContextDelete),
				
				("MacroInsert", gtk.STOCK_GO_FORWARD, _("Paste Definition"), None, None, self._macros.onMacroContextInsert),
				("MacroEdit", gtk.STOCK_EDIT, _("Edit..."), None, None, self._macros.onMacroContextEdit),
				("MacroDelete", gtk.STOCK_DELETE, _("Delete"), None, None, self._macros.onMacroContextDelete)
		])
		self._manager.insert_action_group(self._action_group, -1)
		self._ui_id = self._manager.add_ui_from_file(const.PATH + "/ui.xml")
		
		self._manager.get_widget("/MenuBar/SearchMenu/FindInDVIMenu").set_sensitive(False)
		
		# insert the toolbar
		box = self._window.get_children()[0]
		self._toolbar = self._manager.get_widget("/LaTeXToolbar")
		box.pack_start(self._toolbar, False)
		box.reorder_child(self._toolbar, 2)
		self._toolbar.set_style(gtk.TOOLBAR_ICONS)
		self._toolbar.show()
		
		# insert the toolbar CheckMenuItem
		self._menu_view_toolbar = gtk.CheckMenuItem(_("LaTeX toolbar"))
		self._menu_view_toolbar.set_active(True)
		self._manager.get_widget("/MenuBar/ViewMenu").get_submenu().insert(self._menu_view_toolbar, 2)
		self._menu_view_toolbar.show()
		self._menu_view_toolbar.connect("toggled", self._on_view_toolbar_toggled)
		
		# set custom icons
		for path in CUSTOM_ICONS.keys():
			widget = self._manager.get_widget(path)
			image = gtk.Image()
  			image.set_from_file(const.PATH + CUSTOM_ICONS[path])
  			image.show()
			if type(widget) is gtk.ImageMenuItem:
				widget.set_image(image)
			elif type(widget) is gtk.ToolButton:
				widget.set_icon_widget(image)
		
		# insert typeface SmartMenuToolButton
		self._typeface_button = SmartMenuToolButton(_("Typeface"), self._manager.get_widget("/TypefacePopup"), 2)
		self._typeface_button.show()
		self._toolbar.insert(self._typeface_button, 6)
		
		# insert structure SmartMenuToolButton
		self._structure_button = SmartMenuToolButton(_("Headline"), self._manager.get_widget("/StructurePopup"), 3)
		self._structure_button.show()
		self._toolbar.insert(self._structure_button, 15)
		
		# insert math MenuToolButton
		image = gtk.Image()
		image.set_from_file(const.PATH + "/pixmaps/math.png")
		self._math_button = gtk.MenuToolButton(image, _("Mathematical"))
		self._math_button.set_menu(self._manager.get_widget("/MathMenu"))
		self._math_button.connect("clicked", lambda a: self.get_document().insert(("$", "$")))
		self._math_button.show()
		self._toolbar.insert(self._math_button, 17)
		
		# insert accents SmartMenuToolButton
		self._accent_button = SmartMenuToolButton(_("Accent"), self._manager.get_widget("/AccentPopup"))
		self._accent_button.show()
		self._toolbar.insert(self._accent_button, 21)
		
		# insert compile SmartMenuToolButton
		self._compile_button = SmartMenuToolButton(_("Compile"), self._manager.get_widget("/CompilePopup"))
		self._compile_button.show()
		self._toolbar.insert(self._compile_button, 26)
		
		self._menu_view_toolbar.set_active(self._prefs.setdefault("/toolbar-visible", True))
	
	def _on_view_toolbar_toggled(self, checkmenuitem):
		"""
		Shows or hides the toolbar
		"""
		if checkmenuitem.get_active():
			self._toolbar = self._manager.get_widget("/LaTeXToolbar").show()
		else:
			self._toolbar = self._manager.get_widget("/LaTeXToolbar").hide()
	
	def _insert_macro(self, action):
		environment = self.get_document().environment
		if environment == const.MATH:
			macro = ACTIONS_TO_MACROS[action.get_name()][1]
		else:
			macro = ACTIONS_TO_MACROS[action.get_name()][0]
		if not macro:
			return
		package = macro.get_package()
		if package:
			self.get_document().requirePackages([package])
		self.get_document().insert(tuple(macro))
	
	def _runQuickstartDialog(self):
		if not self._quickstartDialog:
			from QuickstartDialog import QuickstartDialog
			self._quickstartDialog = QuickstartDialog(self)
		if self._quickstartDialog.run() == gtk.RESPONSE_OK:
			document = self.get_document()
			if len(document.getString()) or document.getFilename():
				question = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, 
						gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, _("Should I create a new gedit document?"))
				if question.run() == gtk.RESPONSE_YES:
					self._window.create_tab(True)
					document = self.get_document()
				question.destroy()
			document.insert(self._quickstartDialog.getSource())
			document.enableHighlight()
			document.update()
		self._quickstartDialog.hide()
	
	def _insertGraphics(self):
		if not self._graphicsDialog:
			from GraphicsDialog import GraphicsDialog
			dpi = self._prefs.setdefault("/dpi", 96)
			self._graphicsDialog = GraphicsDialog(self._window, dpi)
		if self._graphicsDialog.run() == gtk.RESPONSE_OK:
			self.get_document().insert(self._graphicsDialog.source)
			self.get_document().requirePackages(["graphicx"])
		self._graphicsDialog.hide()
	
	def _insertTable(self):
		if not self._tableDialog:
			from TableDialog import TableDialog
			self._tableDialog = TableDialog()
		if self._tableDialog.run(self.get_document().environment) == gtk.RESPONSE_OK:
			self.get_document().insert(self._tableDialog.source)
			if self._tableDialog.package:
				self.get_document().requirePackages([self._tableDialog.package])
		self._tableDialog.hide()
	
	def _renderToImage(self):
		if not self._renderImageDialog:
			from RenderImageDialog import RenderImageDialog
			self._renderImageDialog = RenderImageDialog()
		if self._renderImageDialog.run() == gtk.RESPONSE_OK:
			if not self._backend:
				self._backend = Backend()
				self._backendDialog = BackendDialog(self._backend, self)
			self._backend.setImageParameters(self._renderImageDialog.pngMode, self._renderImageDialog.aaFactor, 
					self._renderImageDialog.resolution, self._renderImageDialog.boundingBox)
			self._compileDocument(self._renderImageDialog.format)
		self._renderImageDialog.hide()
	
	def _compileDocument(self, format):
		# check if rubber is installed
		if os.system("rubber --version $2>/dev/null"):
			message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, None)
			message.set_markup("The LaTeX plugin requires <b>rubber</b> for automated document compiling.\nThis tool can be downloaded from <u>rubber.sourceforge.net</u>.")
			message.run()
			message.destroy()
			return
			
		# create a Backend and BackendDialog instance if neccessary
		if not self._backend:
			from backend import RubberBackend, BackendDialog
			self._backend = RubberBackend()
			self._backendDialog = BackendDialog(self._backend, self)
		
		document = self.get_document()
		filename = document.getFilename()
		if filename:
			# save the document (if it has been changed since the last save)
			self._saveAction.activate()
			# compile it
			if not self._backend.compile(format, filename=filename):
				# compile() only returns False if the master document could not be found
				message = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, None)
				message.set_markup("The current document seems to be included by another one, which could not be found though.\n\n<b>Hint 1:</b> The parent document and all included child documents must be in the same directory.\n<b>Hint 2:</b> \\include commands can not be nested.")
				message.run()
				message.destroy()
				return
		else:
			self._backend.compile(format, string=document.getString())
		
		## enable/disable DVI search
		document.dviCompiled = (format == const.DVI)
	
	def _performDVISearch(self):
		if not self._backend:
			return
		self._backend.findLine(self.get_document().getLine() + 1)
	
	def _performPrimarySearch(self):
		""" Searches the active document for the string from the primary clipboard """
		text = self._clipboard.wait_for_text()
		# Here we should be doing some cleaning-up
		# Remove hyphenation in mid-word at line end
		text = text.replace('-\n', '')
		# Replace CRs with blanks and elim needless blanks
		text = text.replace('\n', ' ').strip()
		# Double quotes
		text = text.replace('\xe2\x80\x9c', '``')
		text = text.replace('\xe2\x80\x9d', "''")
		# Ligatures in UTF-8 encoding
		# FIXME: can we use text.translate() ?
		text = text.replace('\xef\xac\x80', 'ff')
		text = text.replace('\xef\xac\x81', 'fi')
		text = text.replace('\xef\xac\x82', 'fl')
		text = text.replace('\xef\xac\x83', 'ffi')
		text = text.replace('\xef\xac\x84', 'ffl')
		self.get_document().findString(text)
	
	def _remove_ui(self):
		""" Removes the plugin's ui """
		self._window.get_side_panel().remove_item(self._symbols)
		self._window.get_side_panel().remove_item(self._structure)
		self._window.get_side_panel().remove_item(self._macros)
		self._toolbar = None
		self._menu_view_toolbar.destroy()
		self._manager.remove_ui(self._ui_id)
		self._manager.remove_action_group(self._action_group)
		self._manager.ensure_update()
	
	def deactivate(self):
		""" Removes the ui and destroys some objects """
		self._remove_ui()
		self._window = None
		self._plugin = None
		self._action_group = None
		self._manager = None
		
	def update_ui(self):
		""" Deactivate the ui if there is no document """
		active_view = self._window.get_active_view()
		
		self._typeface_button.set_sensitive(bool(active_view))
		self._structure_button.set_sensitive(bool(active_view))
		self._math_button.set_sensitive(bool(active_view))
		self._accent_button.set_sensitive(bool(active_view))
		self._compile_button.set_sensitive(bool(active_view))
		self._action_group.set_sensitive(bool(active_view))
		
		if active_view:
			## enable/disable DVI search menu entry
			document = self.get_document()
			if document:
				self._manager.get_widget("/MenuBar/SearchMenu/FindInDVIMenu").set_sensitive(self.dviSearchEnabled and document.dviCompiled)
		
			if not active_view in self._document_wrappers:
				# the GeditView is not wrapped yet
				wrapper = DocumentWrapper(active_view, self)
				self._document_wrappers[active_view] = wrapper
			else:
				# the GeditView is already wrapped - connect to the DocumentWrapper
				# and activate it
				wrapper = self._document_wrappers[active_view]
				if wrapper is self._last_wrapper:
					wrapper.activate()
					self._update_smart_widgets()
					return
			# disconnect the last wrapper
			if self._last_wrapper:
				self._last_wrapper.disconnect(self._wrapper_connections["environment-changed"])
				self._last_wrapper.disconnect(self._wrapper_connections["structure-updated"])
			# connect to the signals of the wrapper
			self._wrapper_connections["environment-changed"] = wrapper.connect("environment-changed", lambda a: self._update_smart_widgets())
			self._wrapper_connections["structure-updated"] = wrapper.connect("structure-updated", lambda a: self._structure.viewStructure.expand_all())
			# pass the structure
			self._structure.viewStructure.set_model(wrapper.modelStructure)
			self._structure.viewStructure.expand_all()
			# labels
			self._structure.viewLabels.set_model(wrapper.modelLabelsAndBibs)
			self._macros.viewNewcommands.set_model(wrapper.modelNewcommands)
			
			self._last_wrapper = wrapper
			self._update_smart_widgets()
		else:
			# pass the structure
			self._structure.viewStructure.set_model(None)
			# labels
			self._structure.viewLabels.set_model(None)
			self._macros.viewNewcommands.set_model(None)
	
	def _cleanup(self):
		""" Stores preferences """
		self._prefs["/macros-paned-position"] = self._macros.get_position()
		self._prefs["/structure-paned-position"] = self._structure.get_position()
		self._prefs["/toolbar-visible"] = self._menu_view_toolbar.get_active()
	
	def _update_smart_widgets(self):
		""" (de)activates widgets that are only defined in special environments """
		environment = self.get_document().environment
		for path in SMART_WIDGETS:
			if environment == const.UNKNOWN:
				self._manager.get_widget(path).set_sensitive(True)
			else:
				if environment & SMART_WIDGETS[path]:
					self._manager.get_widget(path).set_sensitive(True)
				else:
					self._manager.get_widget(path).set_sensitive(False)
	
class SmartMenuToolButton(gtk.MenuToolButton):
	""" A custom MenuToolButton that switches the icon and action of its button 
		to the last selected ones """
	activeItem = None
	
	def __init__(self, label, menu, index=0):
		item = menu.get_children()[index]
		self.image = gtk.Image()
		self.image.set_from_pixbuf(item.get_image().get_pixbuf())
		self.image.show()
		self.activeItem = item
		
		gtk.MenuToolButton.__init__(self, self.image, label)
		self.button = self.get_children()[0].get_children()[0]
		
		self.__set_menu(menu)
	
	def __set_menu(self, menu):
		children = menu.get_children()
		for child in children:
			child.connect("activate", self.__onItemActivated)
		gtk.MenuToolButton.set_menu(self, menu)
		self.connect("clicked", self.__onButtonClicked)
		
	def __onItemActivated(self, item):
		""" called when a menu item is activated """
		self.image.set_from_pixbuf(item.get_image().get_pixbuf())
		self.button.set_image(self.image)
		self.activeItem = item

	def __onButtonClicked(self, button):
		""" called when the MenuToolButton is clicked """
		if self.activeItem:
			self.activeItem.emit("activate")

class LaTeXPlugin(gedit.Plugin):
	TAG = "LaTeXPluginInstance"
	
	def __init__(self):
		gedit.Plugin.__init__(self)
	
	def _get_instance(self, window):
		return window.get_data(self.TAG)
	
	def _set_instance(self, window, instance):
		window.set_data(self.TAG, instance)
	
	def activate(self, window):
		"""
		called when a new gedit window is created
		"""
		self._set_instance(window, LaTeXPluginInstance(self, window))
	
	def deactivate(self, window):
		"""
		called when a window is destroyed
		"""
		self._get_instance(window).deactivate()
		self._set_instance(window, None)
	
	def update_ui(self, window):
		"""
		called when the window is updated (tab change, menu update etc.)
		"""
		self._get_instance(window).update_ui()
	
	def create_configure_dialog(self):
		"""
		called when the plugin is to be configured (must return a gtk.Window)
		"""
		from config import ConfigDialog
		return ConfigDialog()
