# -*- coding: utf-8 -*-

# completion.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import const
import gtk

class CompletionProvider:
	lastPrefix = None
	targets = []
	
	def __init__(self, builtinTargets, modelNewcommands, modelLabelsBibitems, modelPackages):
		self.builtinTargets = builtinTargets
		self.modelNewcommands, self.modelLabelsBibitems, self.modelPackages = modelNewcommands, modelLabelsBibitems, modelPackages
		
	def update(self):
		# create \refS and \pagerefS from the \labelS and \citeS from the \bibitemS
		targets = self.builtinTargets \
				+ ["ref{%s}" % row[0] for row in self.modelLabelsBibitems if row[1] == const.LABEL] \
				+ ["pageref{%s}" % row[0] for row in self.modelLabelsBibitems if row[1] == const.LABEL] \
				+ ["cite{%s}" % row[0] for row in self.modelLabelsBibitems if row[1] == const.BIBITEM] \
				+ [u"%s\t\u2192 %s" % (row[0][1:], row[1]) for row in self.modelNewcommands]
		for row in self.modelPackages:
			# load package file
			try:
				targets += open("%s/completion/%s" % (const.PATH, row[0])).read().splitlines()
			except IOError:
				continue
		targets.sort()
		# split targets at \t
		self.targets = []
		for splitTarget in [target.split("\t") for target in targets]:
			if len(splitTarget) > 1:
				self.targets.append((splitTarget[0], splitTarget[1]))
			else:
				self.targets.append((splitTarget[0], ""))
	
	def complete(self, prefix):
		""" Attempts to complete <prefix> using the loaded target items """
		self.lastPrefix = prefix
		return [target for target in self.targets if target[0].startswith(prefix)]
		

class CompletionPopup(gtk.Window):
	def __init__(self):
		gtk.Window.__init__(self, gtk.WINDOW_POPUP)
		self.connect("hide", self.onHide)
		self.connect("show", self.onShow)
		self.connect("configure-event", self.onMove)
		
		self.hintPopup = HintPopup()
		
		self.model = gtk.ListStore(str, str)	# target, hint
		
		self.treeView = gtk.TreeView(self.model)
		self.treeView.insert_column_with_attributes(-1, "", gtk.CellRendererText(), text=0)
		self.treeView.get_column(0).set_min_width(100)
		self.treeView.set_enable_search(False)
		self.treeView.set_headers_visible(False)
		self.treeView.connect_after("cursor-changed", self.onCursorChanged)
		
		scroll = gtk.ScrolledWindow()
		scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		scroll.add(self.treeView)
		scroll.set_size_request(-1, 150)
		
		frame = gtk.Frame()
		frame.set_shadow_type(gtk.SHADOW_OUT)
		frame.add(scroll)
		
		self.add(frame)
	
	def onMove(self, widget, event):
		""" Update the hint's position """
		x, y = self.get_position()
		width = self.get_size()[0]
		path, column = self.treeView.get_cursor()
		rect = self.treeView.get_cell_area(path, column)
		self.hintPopup.move(x + width + 5, y + rect.y)
	
	def onShow(self, widget):
		self.showHint()
	
	def onHide(self, widget):
		self.hintPopup.hide()
	
	def onCursorChanged(self, treeView):
		if not self.get_property("visible"):
			self.hintPopup.hide()
			return
		self.showHint()
	
	def showHint(self):
		listStore, listIter = self.treeView.get_selection().get_selected()
		hint = listStore.get_value(listIter, 1)
		if not hint:
			self.hintPopup.hide()
			return
			
		x, y = self.get_position()
		width = self.get_size()[0]
		
		path, column = self.treeView.get_cursor()
		rect = self.treeView.get_cell_area(path, column)
		
		self.hintPopup.popup(hint, x + width + 5, y + rect.y)
		

class HintPopup(gtk.Window):
	def __init__(self):
		gtk.Window.__init__(self, gtk.WINDOW_POPUP)
		self.label = gtk.Label()
		self.label.set_padding(5, 2)
		frame = gtk.Frame()
		frame.set_shadow_type(gtk.SHADOW_OUT)
		frame.add(self.label)
		self.add(frame)
	
	def popup(self, string, x, y):
		self.resize(1, 1)
		self.label.set_markup(string)
		self.move(x, y)
		self.show_all()
		
