# -*- coding: utf-8 -*-

# macros.py
# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2006 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

import gtk
import const

# TODO:
# * \newcommand: context menu with "Go to definition", "Insert command" and "Delete"
# * user-defined macros should be stored under internal keys (maybe md5 of the definition) 
#   and their name should be stored in /name

(ADD_MACRO, EDIT_MACRO, ADD_NEWCOMMAND, EDIT_NEWCOMMAND) = range(4)

class Macros(gtk.VPaned):
	_macroDialog = None
	reArgRef = None		# compiled on demand
	
	def __init__(self, instance):
		self._instance = instance
		gtk.VPaned.__init__(self)
		
		## newcommands
		self.viewNewcommands = gtk.TreeView()
		self.viewNewcommands.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), markup=2)
		self.viewNewcommands.insert_column_with_attributes(-1, _("Definition"), gtk.CellRendererText(), text=1)
		self.viewNewcommands.set_headers_visible(False)
		self.viewNewcommands.connect("row-activated", self.__onNewcommandSelected)
		self.viewNewcommands.connect("button-press-event", self.__onNewcommandContext)
		
		scrNewcommands = gtk.ScrolledWindow()
		scrNewcommands.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);
		scrNewcommands.set_shadow_type(gtk.SHADOW_IN)
		scrNewcommands.add(self.viewNewcommands)
		
		btnAddNewcommand = gtk.Button(_("Add New Command"))
		btnAddNewcommand.set_relief(gtk.RELIEF_NONE)
		btnAddNewcommand.set_image(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_MENU))
		btnAddNewcommand.connect("clicked", lambda a: self.__addNewcommand())
		
		vboxNewcommands = gtk.VBox()
		vboxNewcommands.pack_start(scrNewcommands, True)
		vboxNewcommands.pack_start(btnAddNewcommand, False)
		
		## macro library
		self.storeMacros = gtk.ListStore(str, str, int, str, str, str)	# name, definition, scope, checksum, packages (,-sep)
		self.__updateMacros()
		
		self.viewMacros = gtk.TreeView(self.storeMacros)
		self.viewMacros.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), markup=5)
		self.viewMacros.insert_column_with_attributes(-1, _("Definition"), gtk.CellRendererText(), text=1)
		self.viewMacros.set_headers_visible(False)
		self.viewMacros.connect("row-activated", self.__onMacroSelected)
		self.viewMacros.connect("button-press-event", self.__onMacroContext)
	
		scrMacros = gtk.ScrolledWindow()
		scrMacros.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrMacros.set_shadow_type(gtk.SHADOW_IN)
		scrMacros.add(self.viewMacros)
		
		tt = gtk.Tooltips()
		
		btnAddMacro = gtk.ToolButton(gtk.STOCK_ADD)
		btnAddMacro.set_tooltip(tt, "Add macro...")
		btnAddMacro.connect("clicked", lambda a: self.__addMacro())
		
		btnImportMacros = gtk.ToolButton(gtk.STOCK_OPEN)
		btnImportMacros.set_tooltip(tt, "Import LaTeX style file...")
		btnImportMacros.connect("clicked", lambda a: self.__importMacros())
		
		btnExportMacros = gtk.ToolButton(gtk.STOCK_CONVERT)
		
		tMacros = gtk.Toolbar()
		tMacros.set_tooltips(True)
		tMacros.insert(btnAddMacro, -1)
		tMacros.insert(btnImportMacros, -1)
		tMacros.insert(btnExportMacros, -1)
		
		vboxMacros = gtk.VBox()
		vboxMacros.pack_start(scrMacros, True)
		vboxMacros.pack_start(tMacros, False)
		
		self.add1(vboxNewcommands)
		self.add2(vboxMacros)
		
		self.show_all()
		self._instance._prefs.add_notify(lambda *a: self.__updateMacros(), "/user-defined-macros")
	
	def __importMacros(self):
		filename, macros = None, None
		fileChooser = gtk.FileChooserDialog(title=_("Import Style File"), buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		if fileChooser.run() == gtk.RESPONSE_OK:
			filename = fileChooser.get_filename()
		fileChooser.destroy()
		if filename:
			importDialog = ImportDialog()
			if importDialog.run(filename) == gtk.RESPONSE_OK:
				macros = importDialog.getMacros()
			importDialog.destroy()
			if macros:
				import md5
				for macro in macros:
					checksum = md5.new(macro[1]).hexdigest()
					subdir = "/user-defined-macros/" + checksum
					if self._instance._prefs.dir_exists(subdir):
						pass
						#print "Error: A macro with that definition already exists."
					else:
						self._instance._prefs.add_dir(subdir)
						self._instance._prefs[subdir + "/name"] = macro[0]
						self._instance._prefs[subdir + "/definition"] = macro[1]
					
	def __addNewcommand(self):
		""" Shows the dialog for adding a newcommand """
		if not self._macroDialog:
			self._macroDialog = MacroDialog()
		if self._macroDialog.run(ADD_NEWCOMMAND) == gtk.RESPONSE_OK:
			name = self._macroDialog.getName()
			if name[0] != "\\":
				name = "\\" + name
			definition = self._macroDialog.getDefinition()
			self._instance.get_document().insertNewcommand(name, definition)
		self._macroDialog.hide()
	
	def __addMacro(self):
		""" Shows the dialog for adding a user-defined macro """
		if not self._macroDialog:
			self._macroDialog = MacroDialog()
		if self._macroDialog.run(ADD_MACRO) == gtk.RESPONSE_OK:
			import md5
			checksum = md5.new(self._macroDialog.definition).hexdigest()
			subdir = "/user-defined-macros/" + checksum
			if self._instance._prefs.dir_exists(subdir):
				print "Error: A macro with that definition already exists."
			else:
				self._instance._prefs.add_dir(subdir)
				self._instance._prefs[subdir + "/name"] = self._macroDialog.getName()
				self._instance._prefs[subdir + "/scope"] = self._macroDialog.getScope()
				self._instance._prefs[subdir + "/definition"] = self._macroDialog.getDefinition()
				self._instance._prefs[subdir + "/packages"] = self._macroDialog.getPackages()
		self._macroDialog.hide()
	
	def __updateMacros(self):
		""" Search the configuration for user-defined macros """
		self.storeMacros.clear()
		for path in self._instance._prefs.get_subdirs("/user-defined-macros"):
			checksum = path.split("/")[-1]
			name = self._instance._prefs["/user-defined-macros/%s/name" % checksum]
			definition = self._instance._prefs["/user-defined-macros/%s/definition" % checksum]
			scope = self._instance._prefs.setdefault("/user-defined-macros/%s/scope" % checksum, 0)
			packages = ",".join(self._instance._prefs.setdefault("/user-defined-macros/%s/packages" % checksum, []))
			if name and definition:
				self.storeMacros.append([name, definition, scope, checksum, packages, "<b>%s</b>" % name])
		
	def __insertMacro(self, macro, scope, packages):
		environment = self._instance.get_document().environment
		# consider the scope of the macro
		if (scope == const.GENERAL and environment == const.MATH) or (scope == const.MATH and environment != const.MATH):
			macro = "$ %s $" % macro
		# insert the macro
		self._instance.get_document().insert(macro)
		# check required packages
		if len(packages):
			self._instance.get_document().requirePackages(packages.split(","))
	
	def __onMacroSelected(self, treeview, path, column):	
		""" Inserts a user-defined macro """
		liststore = treeview.get_model()
		macro = liststore.get_value(liststore.get_iter(path), 1)
		scope = liststore.get_value(liststore.get_iter(path), 2)
		packages = liststore.get_value(liststore.get_iter(path), 4)
		self.__insertMacro(macro, scope, packages)
	
	def onMacroContextInsert(self, action):
		""" Inserts a user-defined macro (called from the context menu) """
		liststore, listiter = self.viewMacros.get_selection().get_selected()
		macro = liststore.get_value(listiter, 1)
		scope = liststore.get_value(listiter, 2)
		packages = liststore.get_value(listiter, 4)
		self.__insertMacro(macro, scope, packages)
	
	def __onMacroContext(self, treeview, event):
		""" Displays the context menu for a user-defined macro """
		if event.button == 3:
			x, y = int(event.x), int(event.y)
			pathTuple = treeview.get_path_at_pos(x, y)
            
			if pathTuple is not None:
				path = pathTuple[0]
				treeview.grab_focus()
				treeview.set_cursor(path)
		
			selection = treeview.get_selection()
			if not selection.count_selected_rows():
				return
			self._instance._manager.get_widget("/MacroPopup").popup(None, None, None, event.button, event.time)
	
	def onMacroContextEdit(self, action):
		""" Shows the dialog for editing a user-defined macro """
		liststore, listiter = self.viewMacros.get_selection().get_selected()
		checksum = liststore.get_value(listiter, 3)
		
		if not self._macroDialog:
			self._macroDialog = MacroDialog()
		
		## load the macro to be edited
		name = self._instance._prefs["/user-defined-macros/%s/name" % checksum]
		definition = self._instance._prefs["/user-defined-macros/%s/definition" % checksum]
		scope = self._instance._prefs["/user-defined-macros/%s/scope" % checksum]
		packages = self._instance._prefs.setdefault("/user-defined-macros/%s/packages" % checksum, [])
		self._macroDialog.load(name, definition, scope, packages)
		
		if self._macroDialog.run(EDIT_MACRO) == gtk.RESPONSE_OK:
			## save changes to the macro
			subdir = "/user-defined-macros/" + checksum
			self._instance._prefs[subdir + "/name"] = self._macroDialog.getName()
			self._instance._prefs[subdir + "/scope"] = self._macroDialog.getScope()
			self._instance._prefs[subdir + "/definition"] = self._macroDialog.getDefinition()
			self._instance._prefs[subdir + "/packages"] = self._macroDialog.getPackages()
		self._macroDialog.hide()
		
	def onMacroContextDelete(self, action):
		"""
		Deletes a user-defined macro from the configuration
		"""
		liststore, listiter = self.viewMacros.get_selection().get_selected()
		checksum = liststore.get_value(listiter, 3)
		# delete all subentries of the macro
		self._instance._prefs.unset("/user-defined-macros/%s/name" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/definition" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/scope" % checksum)
		self._instance._prefs.unset("/user-defined-macros/%s/packages" % checksum)
	
	def __onNewcommandSelected(self, treeview, path, column):
		"""
		Inserts a newcommand into the document
		(called on 'row-activated' of the listview)
		"""
		liststore = treeview.get_model()
		command = liststore.get_value(liststore.get_iter(path), 0)
		self._instance.get_document().insert(command)
	
	def __onNewcommandContext(self, treeview, event):
		"""
		Displays the context menu for newcommand entries
		"""
		if event.button == 3:
			x, y = int(event.x), int(event.y)
			pathTuple = treeview.get_path_at_pos(x, y)
            
			if pathTuple is not None:
				path = pathTuple[0]
				treeview.grab_focus()
				treeview.set_cursor(path)
		
			selection = treeview.get_selection()
			if not selection.count_selected_rows():
				return
			self._instance._manager.get_widget("/NewcommandPopup").popup(None, None, None, event.button, event.time)
	
	def onNewcommandContextJump(self, action):
		""" Jumps to the definition of a newcommand """
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		string = liststore.get_value(listiter, 3)
		self._instance.get_document().findString(string)
	
	def onNewcommandContextInsert(self, action):
		"""
		Inserts a newcommand into the document
		(called from the context menu)
		"""
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		command = liststore.get_value(listiter, 0)
		self._instance.get_document().insert(command)
	
	def onNewcommandContextEdit(self, action):
		""" Shows the dialog for editing a newcommand """
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		name = liststore.get_value(listiter, 0)
		definition = liststore.get_value(listiter, 1)
		
		if not self._macroDialog:
			self._macroDialog = MacroDialog()
		
		## load the newcommand to be edited
		self._macroDialog.load(name, definition)
		
		if self._macroDialog.run(EDIT_NEWCOMMAND) == gtk.RESPONSE_OK:
			## save changes to the newcommand
			self._instance.get_document().changeNewcommand(name, self._macroDialog.getName(), self._macroDialog.getDefinition())
		self._macroDialog.hide()
	
	def onNewcommandContextDelete(self, action):
		liststore, listiter = self.viewNewcommands.get_selection().get_selected()
		name = liststore.get_value(listiter, 0)
		self._instance.get_document().deleteNewcommand(name)


class ImportDialog(gtk.Dialog):
	"""
	Dialog for importing an .sty-file
	"""
	def __init__(self):
		import re
		self.reNewcommand = re.compile(r"\\newcommand\{\\(.*?)\}(\[[1-9]?\])?\{(.*)\}")
	
		gtk.Dialog.__init__(self, _("Import Style File"), None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_has_separator(False)
		self.vbox.set_spacing(5)	
			
		self.listStore = gtk.ListStore(bool, str, str)
		self.treeView = gtk.TreeView(self.listStore)
		
		cellToggle = gtk.CellRendererToggle()
		cellToggle.set_property("activatable", True)
		cellToggle.connect("toggled", self.__onRowToggled)
		columnToggle = gtk.TreeViewColumn("Import", cellToggle)
		columnToggle.add_attribute(cellToggle, "active", 0)
		self.treeView.append_column(columnToggle)
		
		self.treeView.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), text=1)
		self.treeView.insert_column_with_attributes(-1, _("Definition"), gtk.CellRendererText(), text=2)
		
		scroll = gtk.ScrolledWindow()
		scroll.set_border_width(5)
		scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC);
		scroll.set_shadow_type(gtk.SHADOW_IN)
		scroll.add(self.treeView)
		
		self.vbox.add(scroll)
		
		self.set_size_request(500, 300)
		self.show_all()
	
	def __onRowToggled(self, renderer, path):
		self.listStore[path][1] = not self.listStore[path][1]
	
	def run(self, filename):
		self.listStore.clear()
		for match in self.reNewcommand.finditer(open(filename).read()):
			self.listStore.append([True, match.group(1), match.group(3)])
		
		return gtk.Dialog.run(self)
	
	def getMacros(self):
		return [(row[1], row[2]) for row in self.listStore if row[0]]

class MacroDialog(gtk.Dialog):
	""" 
	Dialog for adding/editing a user-defined macro and adding a new command 
	"""
	mode = None
	
	def __init__(self):
		import gtksourceview, pango
		from PreviewArea import PreviewArea
	
		gtk.Dialog.__init__(self, None, None, gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		## name
		self.entryName = gtk.Entry()
		self.entryName.show()
		self.entryName.connect("changed", self.__onNameChanged)
		lblName = gtk.Label(_("_Name:"))
		lblName.show()
		lblName.set_use_underline(True)
		lblName.set_alignment(0, .5)
		lblName.set_mnemonic_widget(self.entryName)
		
		## definition
		manager = gtksourceview.SourceLanguagesManager()
		language = manager.get_language_from_mime_type("text/x-tex")
		self.bufDef = gtksourceview.SourceBuffer()
		self.bufDef.set_language(language)
		self.bufDef.set_highlight(True)
		self.bufDef.connect("changed", self.__onDefChanged)
		
		srcDefinition = gtksourceview.SourceView(self.bufDef)
		srcDefinition.show()
		srcDefinition.set_wrap_mode(gtk.WRAP_WORD)
		srcDefinition.modify_font(pango.FontDescription("Monospace"))
		
		scrDefinition = gtk.ScrolledWindow()
		scrDefinition.show()
		scrDefinition.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC);
		scrDefinition.set_shadow_type(gtk.SHADOW_IN)
		scrDefinition.add(srcDefinition)
		
		lblDef = gtk.Label(_("_Definition:"))
		lblDef.show()
		lblDef.set_use_underline(True)
		lblDef.set_mnemonic_widget(scrDefinition)
		lblDef.set_alignment(0, 0)
		
		## packages
		self.cmbPackage = gtk.combo_box_entry_new_text()
		self.cmbPackage.show()
		for item in ["amsmath", "amssymb"]:
			self.cmbPackage.append_text(item)
		
		lblPackages = gtk.Label(_("Required _Packages:"))
		lblPackages.show()
		lblPackages.set_use_underline(True)
		lblPackages.set_mnemonic_widget(self.cmbPackage)
		lblPackages.set_alignment(0, .5)
		
		self.storePackages = gtk.ListStore(str)
		self.viewPackages = gtk.TreeView(self.storePackages)
		self.viewPackages.show()
		self.viewPackages.insert_column_with_attributes(-1, _("Name"), gtk.CellRendererText(), text=0)
		self.viewPackages.set_headers_visible(False)
		scrPackages = gtk.ScrolledWindow()
		scrPackages.show()
		scrPackages.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		scrPackages.set_shadow_type(gtk.SHADOW_IN)
		scrPackages.add(self.viewPackages)
		
		btnAddPackage = gtk.Button()
		btnAddPackage.show()
		btnAddPackage.set_relief(gtk.RELIEF_NONE)
		image = gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON)
		image.show()
		btnAddPackage.add(image)
		btnAddPackage.connect("clicked", lambda a: self.__addPackage())
		
		btnRemPackage = gtk.Button()
		btnRemPackage.show()
		btnRemPackage.set_relief(gtk.RELIEF_NONE)
		image = gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_BUTTON)
		image.show()
		btnRemPackage.add(image)
		btnRemPackage.connect("clicked", lambda a: self.__removePackage())
		algRemPackage = gtk.Alignment(.5, 0, 1, 0)
		algRemPackage.show()
		algRemPackage.add(btnRemPackage)
		
		tblPackages = gtk.Table()
		tblPackages.show()
		tblPackages.attach(self.cmbPackage, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, False)
		tblPackages.attach(btnAddPackage, 1, 2, 0, 1, False, False)
		tblPackages.attach(scrPackages, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND)
		tblPackages.attach(algRemPackage, 1, 2, 1, 2, False, gtk.FILL)
		
		## scope
		self.cmbScope = gtk.combo_box_new_text()
		self.cmbScope.show()
		for item in [_("Undefined"), _("General"), _("Mathematical"), _("Both")]:
			self.cmbScope.append_text(item)
		self.cmbScope.set_active(0)
		
		lblScope = gtk.Label(_("_Scope:"))
		lblScope.show()
		lblScope.set_use_underline(True)
		lblScope.set_mnemonic_widget(self.cmbScope)
		lblScope.set_alignment(0, .5)
		
		## preview
		self.preview = PreviewArea()	
		self.preview.show()
		scrPreview = gtk.ScrolledWindow()
		scrPreview.show()
		scrPreview.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrPreview.set_shadow_type(gtk.SHADOW_NONE)
		scrPreview.add_with_viewport(self.preview)
		scrPreview.connect("button-press-event", lambda *a: self.renderPreview())
		
		lblPreview = gtk.Label()
		lblPreview.show()
		lblPreview.set_markup(_("Preview:\n<small>Click to reload</small>"))
		lblPreview.set_alignment(0, 0)
		
		table = gtk.Table()
		table.show()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(10)
		table.attach(lblName, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
		table.attach(self.entryName, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, False)
		table.attach(lblDef, 0, 1, 1, 2, gtk.FILL, gtk.FILL)
		table.attach(scrDefinition, 1, 2, 1, 2)
		table.attach(lblPreview, 0, 1, 2, 3, gtk.FILL, gtk.FILL)
		table.attach(scrPreview, 1, 2, 2, 3)
		self.vbox.add(table)
		
		tblAdv = gtk.Table()
		tblAdv.show()
		tblAdv.set_row_spacings(5)
		tblAdv.set_col_spacings(10)
		tblAdv.attach(lblPackages, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
		tblAdv.attach(tblPackages, 1, 2, 0, 1)
		tblAdv.attach(lblScope, 0, 1, 1, 2, gtk.FILL, False)
		tblAdv.attach(self.cmbScope, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
		
		algAdv = gtk.Alignment(.5, .5, 1, 1)
		algAdv.show()
		algAdv.set_padding(0, 0, 20, 5)
		algAdv.add(tblAdv)
		
		self.expander = gtk.Expander(_("<b>_Advanced Options</b>"))
		self.expander.set_use_markup(True)
		self.expander.set_use_underline(True)
		self.expander.add(algAdv)
		self.vbox.pack_start(self.expander, False)
		
		## store a pointer to the OK button
		for child in self.action_area.get_children():
			if self.get_response_for_widget(child) == gtk.RESPONSE_OK:
				child.set_sensitive(False)
				self._button_okay = child
				
		srcDefinition.set_size_request(300, 70)
		
		self.set_default_response(gtk.RESPONSE_OK)
	
	def __addPackage(self):
		self.storePackages.append([self.cmbPackage.get_active_text()])
	
	def __removePackage(self):
		listStore, treeIter = self.viewPackages.get_selection().get_selected()
		try:
			listStore.remove(treeIter)
		except TypeError:
			pass
	
	def load(self, name="", definition="", scope=0, packages=[]):
		self.entryName.set_text(name)
		self.bufDef.set_text(definition)
		self.cmbScope.set_active(scope)
		self.storePackages.clear()
		for package in packages:
			self.storePackages.append([package])
		self.renderPreview()
	
	def run(self, mode):
		self.mode = mode
		self.set_title({ADD_MACRO:_("Add Macro"), EDIT_MACRO:_("Edit Macro"), \
				ADD_NEWCOMMAND:_("Add New Command"), EDIT_NEWCOMMAND:_("Edit New Command")}[mode])
		
		## hide advanced option at newcommands
		if mode in [ADD_NEWCOMMAND, EDIT_NEWCOMMAND]:
			self.expander.hide()
		else:
			self.expander.show()
			
		## reset if neccessary
		if mode in [ADD_NEWCOMMAND, ADD_MACRO]:
			self.__reset()
		
		return gtk.Dialog.run(self)
	
	def __reset(self):
		self.entryName.set_text("")
		self.bufDef.set_text("")
		self.storePackages.clear()
		self.preview.clear()
		self.cmbScope.set_active(0)
	
	def renderPreview(self):
		source = self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter())
		if self.mode == ADD_NEWCOMMAND:
			self.preview.setEscapeRefs(True)
		else:
			if self.cmbScope.get_active() == 2:
				source = "$%s$" % source
		self.preview.setPackages([row[0] for row in self.storePackages])
		self.preview.setSource(source)
		self.preview.render()
	
	def __onNameChanged(self, entry):
		if len(self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter())):
			self._button_okay.set_sensitive(bool(len(self.entryName.get_text())))
	
	def __onDefChanged(self, textbuffer):
		if len(self.entryName.get_text()):
			self._button_okay.set_sensitive(bool(len(textbuffer.get_text(textbuffer.get_start_iter(), textbuffer.get_end_iter()))))
	
	def getName(self):
		return self.entryName.get_text()
	
	def getDefinition(self):
		return self.bufDef.get_text(self.bufDef.get_start_iter(), self.bufDef.get_end_iter())
	
	def getScope(self):
		return self.cmbScope.get_active()
	
	def getPackages(self):
		return [row[0] for row in self.storePackages]
